/* Copyright 2015 Sławomir Lach <slawek@lach.art.pl>
 * Public domain
 */

#include <libgreattao/tao.h>
#include <libgreattao/config.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

struct repo {
  char *repodir;
  struct repo *next;
  char* lastest_version;
} *first_repo;

struct message_window_data {
  int answer;
  void *message_dialog;
};

const struct tao_app_config_option options[2] = {
  {"delay", "delay", "How many seconds this program will wait betweem checks", TAO_TYPE_UINT, '\0', TAO_CONFIG_OPTION_ARG_LONG_DECORATOR},
  {"directory-list", "Directories list", "List of directories of repositories to be checked for searching for updates", TAO_TYPE_FILE_PATH, '\0', TAO_CONFIG_OPTION_ARG_LONG_DECORATOR}
};

struct tao_app_config_result option_parameters[2];

static void exit_from_application_with_error(void *mess1, void *mess2)
{
  exit(1);
}

static void continue_application(void *mess1, void *mess2)
{
  tao_close();
}

static void repo_change_remind_later(void *mess1, struct message_window_data *answer)
{
  answer->answer = 0;
  tao_release_window(answer->message_dialog);
  tao_close();
}

static void repo_change_wait_for_next_update(void *mess1, struct message_window_data *answer)
{
  answer->answer = 1;
  tao_release_window(answer->message_dialog);
  tao_close();
}

static void message_error_with_abort(char *message)
{
  void *message_dialog = tao_new_window("/desktop/dialogs/message_dialog");
  
  tao_add_handler(message_dialog, "/message", NULL, NULL);
  tao_set_hint(message_dialog, "/message", HINT_NAME, "Error");
  tao_set_hint(message_dialog, "/message", HINT_DESCRIPTION, message);
  tao_add_handler(message_dialog, "/:abort", exit_from_application_with_error, NULL);  
  
  tao_handle_events();
}

static int message_repo_changed(struct repo *repo)
{
  struct message_window_data message_window_data;
  char  buffer[1100];
  message_window_data.message_dialog = tao_new_window("/desktop/dialogs/question_dialog");
  
  snprintf(buffer, 1100, "Repository with path %s was changed.", repo->repodir);
  
  tao_add_handler(message_window_data.message_dialog, "/message", NULL, NULL);
  tao_set_hint(message_window_data.message_dialog, "/message", HINT_NAME, "Repository changed - what I should do");
  tao_set_hint(message_window_data.message_dialog, "/message", HINT_DESCRIPTION, buffer);
  tao_add_handler(message_window_data.message_dialog, "/:abort", repo_change_remind_later, &message_window_data);
  
  tao_add_handler(message_window_data.message_dialog, "/actions/remind later", repo_change_remind_later, &message_window_data); 
  tao_add_handler(message_window_data.message_dialog, "/actions/wait for next update", repo_change_wait_for_next_update, &message_window_data);
  
  tao_handle_events();

  return message_window_data.answer;
}

static void translate_file_path(char *filepath)
{
  int char_ommit;
  
  while (*filepath != '\0') {
  
    filepath[-char_ommit] = *filepath;
    if (*filepath == '\n') {
      
      *filepath = '\0';
      break;
    }
    
    if (*filepath == '\\') {
    
      ++char_ommit;
      if (filepath[1] == 'n') {
      
	*filepath = '\n';
	++char_ommit;
      }
      else if (filepath[1] == '\\') {
	
	*filepath = '\\';
	++char_ommit;
      }
      else if (filepath[1] == '\0' || filepath[1] == '\n') {
      
	message_error_with_abort("Backslash before end of file or newline");
      }
    }
    ++filepath;
  }
}

static void load_repo_directories(const char *filename)
{
  char buffer[1000];
  struct repo *curr_repo;
  FILE *repolist = fopen(filename, "r");
  
  if (!repolist) {
    
    message_error_with_abort("Cannot open file");
  }
  
  if (fgets(buffer, 1000, repolist) != NULL) {
  
    curr_repo = (struct repo*) malloc(sizeof(*curr_repo));
    curr_repo->lastest_version = NULL;
    translate_file_path(buffer);
    curr_repo->repodir = strdup(buffer);
    first_repo = curr_repo;
  }
  while (fgets(buffer, 1000, repolist) != NULL) {
  
    curr_repo->next = (struct repo*) malloc(sizeof(*curr_repo));
    curr_repo->next->lastest_version = NULL;
    translate_file_path(buffer);
    curr_repo->next->repodir = strdup(buffer);
    curr_repo = curr_repo->next;
  }
  
  curr_repo->next = NULL;
  fclose(repolist);
}

int main(int argc, char **argv)
{
  int i, j;
  char matched;
  struct repo *curr_repo;
  FILE *to_get_output;
  char *lastest_version;
  char buffer[100];
  
  tao_initialize("repochecker", "", &argc, argv);
  
  tao_get_app_configuration(options, option_parameters, 2, TAO_APP_CONFIG_FROM_FILE | TAO_APP_CONFIG_FROM_LINE | TAO_APP_CONFIG_INTERACTIVE);
  
  if (!option_parameters[0].set || !option_parameters[1].set){
  
    message_error_with_abort("You must specify file path and time delay");
  }
  
  load_repo_directories((char*)(void*)option_parameters[1].result);
  
  if (first_repo == NULL) {
  
    message_error_with_abort("Repolist seems to be empty");
  }
  
  do {

    
    for (curr_repo = first_repo; curr_repo; curr_repo = curr_repo->next) {
    
      if (chdir(curr_repo->repodir) == -1) {
      
        /* We won't to display error messages */
	continue;
      }
      
      to_get_output = popen("svn merge --dry-run -r BASE:HEAD .", "r");
      if (to_get_output == NULL) {

	message_error_with_abort("Unable to start command svnversion");
      }
      matched = 1;
      if (fgets(buffer, 100, to_get_output)) {
	matched = 0;
      }
      fclose(to_get_output);
      lastest_version = NULL;
      
      if (matched == 0) {
	
	i = 0;
	while (buffer[i] != '\0') {
	
	  if (buffer[i] == 'r' && buffer[i+1] >= '0' && buffer[i+1] <= '9') {
	  
	    ++i;
	    break;
	  }
	  ++i;
	}
	
	while (buffer[i] != '\0') {
	  
	  if (buffer[i] == 'r' && buffer[i+1] >= '0' && buffer[i+1] <= '9') {
	    
	    ++i;
	    lastest_version = &buffer[i];
	    while (buffer[i] != ' ') {
	      
	      ++i;
	    }
	    buffer[i] = '\0';
	    break;
	  }
	  ++i;
	}
        
	if (curr_repo->lastest_version && strcmp(curr_repo->lastest_version, lastest_version) == 0) {
	
	  matched = 1;  
	}
      }
      if (!matched) {
	
	j = message_repo_changed(curr_repo);
	if (j == 1) {
	
	  if (curr_repo->lastest_version) {
	  
	    free(curr_repo->lastest_version);
	  }
	  curr_repo->lastest_version = strdup(lastest_version);
	}
      }
    }
    sleep(option_parameters[0].result);
  } while (option_parameters[0].result);
}